-- Hi Emacs, this is -*- mode: vhdl; -*- 
--
-- VT100 Lite Terminal. System On Chip (SoC) with Picoblaze
-- 
-- 2007 Javier Valcarce Garca, javier.valcarce@gmail.com
-- $Id$
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity main is
  port (
    ext_reset : in  std_logic;
    clk50MHz  : in  std_logic;
    R         : out std_logic;
    G         : out std_logic;
    B         : out std_logic;
    hsync     : out std_logic;
    vsync     : out std_logic;
	 tx		  : out std_logic;
	 rx		  : in std_logic
    );
end main;


architecture behavioral of main is

  component embedded_kcpsm3
    port (
      port_id       : out std_logic_vector(7 downto 0);
      write_strobe  : out std_logic;
      read_strobe   : out std_logic;
      out_port      : out std_logic_vector(7 downto 0);
      in_port       : in  std_logic_vector(7 downto 0);
      interrupt     : in  std_logic;
      interrupt_ack : out std_logic;
      reset         : in  std_logic;
      clk           : in  std_logic);
  end component;


  -- TEXT (RAM) MEMORY
  component mem_text
    port (
      clka  : in  std_logic;
      dina  : in  std_logic_vector(07 downto 0);
      addra : in  std_logic_vector(11 downto 0);
      wea   : in  std_logic_vector(00 downto 0);
      douta : out std_logic_vector(07 downto 0);
      clkb  : in  std_logic;
      dinb  : in  std_logic_vector(07 downto 0);
      addrb : in  std_logic_vector(11 downto 0);
      web   : in  std_logic_vector(00 downto 0);
      doutb : out std_logic_vector(07 downto 0));

  end component;

	-- FONT (ROM) MEMORY
  component mem_font
    port (
      clka  : in  std_logic;
      addra : in  std_logic_vector(11 downto 0);
      douta : out std_logic_vector(07 downto 0)
      );
  end component;
  
  
  component vga80x40
    port (
      reset       : in  std_logic;
      clk25MHz    : in  std_logic;
      R           : out std_logic;
      G           : out std_logic;
      B           : out std_logic;
      hsync       : out std_logic;
      vsync       : out std_logic;
		--
      TEXT_A      : out std_logic_vector(11 downto 0);
      TEXT_D      : in  std_logic_vector(07 downto 0);
      FONT_A      : out std_logic_vector(11 downto 0);
      FONT_D      : in  std_logic_vector(07 downto 0);
		--
      ocrx        : in  std_logic_vector(07 downto 0);
      ocry        : in  std_logic_vector(07 downto 0);
      octl        : in  std_logic_vector(07 downto 0)
      );   
  end component;
  
  component uart_rx
	 Port (            serial_in : in std_logic;
                       data_out : out std_logic_vector(7 downto 0);
                    read_buffer : in std_logic;
                   reset_buffer : in std_logic;
                   en_16_x_baud : in std_logic;
            buffer_data_present : out std_logic;
                    buffer_full : out std_logic;
               buffer_half_full : out std_logic;
                            clk : in std_logic);
  end component;
  
  component uart_tx
	 Port (            data_in : in std_logic_vector(7 downto 0);
                 write_buffer : in std_logic;
                 reset_buffer : in std_logic;
                 en_16_x_baud : in std_logic;
                   serial_out : out std_logic;
                  buffer_full : out std_logic;
             buffer_half_full : out std_logic;
                          clk : in std_logic);
  end component;
  
  component clkdivider is
    generic (divideby : natural := 2);
    Port ( clk : in std_logic;
           reset : in std_logic;
           pulseout : out std_logic);
	end component;

  -- kcpsm3
  signal port_id       : std_logic_vector(7 downto 0);
  signal read_strobe   : std_logic;
  signal write_strobe  : std_logic;
  signal interrupt     : std_logic;
  signal interrupt_ack : std_logic;
  signal out_port      : std_logic_vector(7 downto 0);
  signal in_port       : std_logic_vector(7 downto 0);
    
  -- VGA80x40 peripheral
  signal reset       : std_logic; 
  signal clk25MHz    : std_logic; 
  signal clk01kHz    : std_logic; 
  signal ocrx_ce : std_logic;
  signal ocry_ce : std_logic;
  signal octl_ce : std_logic;
  signal ocrx    : std_logic_vector(7 downto 0);
  signal ocry    : std_logic_vector(7 downto 0);
  signal octl    : std_logic_vector(7 downto 0);

  -- Text Buffer RAM Memory Signals, Port B (to CPU core)
  signal text_diA : std_logic_vector(07 downto 0);
  signal text_doA : std_logic_vector(07 downto 0);
  signal text_adA : std_logic_vector(11 downto 0);
  signal text_weA : std_logic_vector(00 downto 0);
  
  signal text_hiA    : std_logic_vector(07 downto 0);
  signal text_loA    : std_logic_vector(07 downto 0);
  signal text_hiA_ce : std_logic;
  signal text_loA_ce : std_logic;


  -- Text Buffer RAM Memory Signals, Port B (to VGA core)
  signal text_diB : std_logic_vector(07 downto 0);
  signal text_doB : std_logic_vector(07 downto 0);
  signal text_adB : std_logic_vector(11 downto 0);
  signal text_weB : std_logic_vector(00 downto 0);
  signal font_doB : std_logic_vector(07 downto 0);
  signal font_adB : std_logic_vector(11 downto 0);
  
  -- UART Signals
  signal baudclk, uart_data_available, uart_rx_full, uart_rx_half, uart_tx_full, uart_tx_half : std_logic;
  signal read_uart_word, write_uart_word : std_logic;
  signal data_from_uartrx : std_logic_vector(7 downto 0);
  signal status_register : std_logic_vector(7 downto 0);
  
begin

  -- Clock divider by /2 (25MHz) and /2^16 (763Hz)
  clkdiv : block
    signal c : unsigned(15 downto 0);
  begin
    c <= c + 1 when rising_edge(clk50MHz);

    clk01kHz <= c(15);                  --763Hz for debouncing flip-flops
    clk25MHz <= c(00);                  --25MHz
    
  end block;
  
  -- Debouncer circuit
  reset <= ext_reset when rising_edge(clk01kHz);
  
  U_CPU : embedded_kcpsm3 port map (
    port_id       => port_id,
    write_strobe  => write_strobe,
    out_port      => out_port,
    read_strobe   => read_strobe,
    in_port       => in_port,
    interrupt     => interrupt,
    interrupt_ack => interrupt_ack,
    reset         => reset,
    clk           => clk50MHz);

  U_VGA : vga80x40 port map (
    reset       => reset,
    clk25MHz    => clk25MHz,
    R           => R,
    G           => G,
    B           => B,
    hsync       => hsync,
    vsync       => vsync,
    TEXT_A      => text_adB,
    TEXT_D      => text_doB,
    FONT_A      => font_adB,
    FONT_D      => font_doB,
	 ocrx        => ocrx,
    ocry        => ocry,
    octl        => octl);
  
  U_TEXT:  mem_text port map (
    clka  => clk50MHz,
    dina  => text_diA,
    addra => text_adA,
    wea   => text_weA,
    douta => text_doA,
    clkb  => clk25MHz,
    dinb  => text_diB,
    addrb => text_adB,
    web   => text_weB,
    doutb => text_doB);
  
  
  U_FONT: mem_font port map (
    clka  => clk25Mhz,
    addra => font_adB,
    douta => font_doB);
	 
  SERIAL_RX : uart_rx port map
	(
		serial_in => rx,
		data_out => data_from_uartrx,
		read_buffer => read_uart_word,
		reset_buffer => reset,
		en_16_x_baud => baudclk,
		buffer_data_present => uart_data_available,
		buffer_full => uart_rx_full,
		buffer_half_full => uart_rx_half,
		clk => clk50MHz);
          	
	SERIAL_TX : uart_tx port map
	 (
		data_in => out_port,
		write_buffer => write_uart_word,
		reset_buffer => reset,
		en_16_x_baud => baudclk,
		serial_out => tx,
		buffer_full => uart_tx_full,
		buffer_half_full => uart_tx_half,
		clk => clk50MHz);
	 
	makebaudclk : clkdivider 
     generic map (	divideby => 27)
	  port map (		clk => clk50MHz, 
						 reset => reset,
					 pulseout => baudclk);
		
  text_diA <= out_port;
  text_adA <= text_hiA(3 downto 0) & text_loA;
  text_weB <= "0";
  text_diB <= (others => '0');
  
  -- OUTPUT IO registers, port_id decoding
  text_weA    <= "1" when port_id = x"20" and write_strobe = '1' else "0"; -- port_id = 32 = 0x20
  text_hiA_ce <= '1' when port_id = x"10" and write_strobe = '1' else '0'; -- port_id = 16 = 0x10
  text_loA_ce <= '1' when port_id = x"08" and write_strobe = '1' else '0'; -- port_id = 08 = 0x08
  ocrx_ce 	  <= '1' when port_id = x"04" and write_strobe = '1' else '0'; -- port_id = 04 = 0x04
  ocry_ce 	  <= '1' when port_id = x"03" and write_strobe = '1' else '0'; -- port_id = 03 = 0x03
  octl_ce 	  <= '1' when port_id = x"02" and write_strobe = '1' else '0'; -- port_id = 02 = 0x02
  
	
  process (reset, clk50MHz)
  begin
    if reset = '1' then
		ocrx <= (others => '0');
		ocry <= (others => '0');
		octl <= (others => '0');
      text_hiA <= (others => '0');
      text_loA <= (others => '0');
    elsif rising_edge(clk50MHz) then
	 
      if text_hiA_ce = '1' then text_hiA <= out_port; end if;
      if text_loA_ce = '1' then text_loA <= out_port; end if;
		if ocrx_ce = '1' then ocrx <= out_port; end if;
		if ocry_ce = '1' then ocry <= out_port; end if;
		if octl_ce = '1' then octl <= out_port; end if;
    end if;
  end process;

  -- handling writes and control signals
  read_uart_word <= '1' when port_id = x"01" and read_strobe='1' else '0';
  write_uart_word <= '1' when port_id = x"01" and write_strobe='1' else '0';
  
  -- misc
  status_register <= "000" & uart_rx_full & uart_rx_half & uart_tx_full & uart_tx_half & uart_data_available;
  
  -- process for picoblaze reading

  readmux: process(port_id)
  begin
	case port_id is
		when x"00" => in_port <= status_register;
		when x"01" => in_port <= data_from_uartrx;
		when x"20" => in_port <= text_doA;
	   when others => in_port <= (others => 'X');
	end case;
  end process readmux;
  
end behavioral;
